# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `IShare.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_ModelF_ice
import omero_ServicesF_ice
import omero_Collections_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module Glacier2
_M_Glacier2 = Ice.openModule('Glacier2')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Included module omero.grid
_M_omero.grid = Ice.openModule('omero.grid')

# Start of module omero
__name__ = 'omero'

# Start of module omero.api
__name__ = 'omero.api'

if 'IShare' not in _M_omero.api.__dict__:
    _M_omero.api.IShare = Ice.createTempClass()
    class IShare(_M_omero.api.ServiceInterface):
        """
        Provides method for sharing - collaboration process for images,
        datasets, projects.
        """
        def __init__(self):
            if Ice.getType(self) == _M_omero.api.IShare:
                raise RuntimeError('omero.api.IShare is an abstract class')

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::api::IShare', '::omero::api::ServiceInterface')

        def ice_id(self, current=None):
            return '::omero::api::IShare'

        def ice_staticId():
            return '::omero::api::IShare'
        ice_staticId = staticmethod(ice_staticId)

        def activate_async(self, _cb, shareId, current=None):
            """
            Turns on the access control lists attached to the given
            share for the current session. Warning: this will slow down
            the execution of the current session for all database
            reads. Writing to the database will not be allowed. If
            share does not exist or is not accessible (non-members) or
            is disabled, then an omero.ValidationException is thrown.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            current -- The Current object for the invocation.
            """
            pass

        def deactivate_async(self, _cb, current=None):
            """
            Turns off the access control lists with the current share.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def getShare_async(self, _cb, shareId, current=None):
            """
            Gets a share as a omero.model.Session with all
            related: omero.model.Annotation comments,
            omero.model.Experimenter members, fully loaded.
            Unlike the other methods on this interface, if the
            sessionId is unknown, does not throw a
            omero.ValidationException.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            current -- The Current object for the invocation.
            """
            pass

        def getMemberCount_async(self, _cb, shareIds, current=None):
            """
            Returns a map from share id to the count of total members
            (including the owner). This is represented by
            omero.model.ShareMember links.
            Arguments:
            _cb -- The asynchronous callback object.
            shareIds -- Not null.
            current -- The Current object for the invocation.
            Throws:
            ValidationException -- if a given share does not exist
            """
            pass

        def getOwnShares_async(self, _cb, active, current=None):
            """
            Gets all owned shares for the current
            omero.model.Experimenter.
            Arguments:
            _cb -- The asynchronous callback object.
            active -- if true, then only shares which can be used for login will be returned. All draft shares (see {@code createShare}) and closed shares (see {@code closeShare}) will be filtered.
            current -- The Current object for the invocation.
            """
            pass

        def getMemberShares_async(self, _cb, active, current=None):
            """
            Gets all shares where current
            omero.model.Experimenter is a member.
            Arguments:
            _cb -- The asynchronous callback object.
            active -- if true, then only shares which can be used for login will be returned. All draft shares (see {@code createShare}) and closed shares (see {@code closeShare}) will be filtered.
            current -- The Current object for the invocation.
            """
            pass

        def getSharesOwnedBy_async(self, _cb, user, active, current=None):
            """
            Gets all shares owned by the given
            omero.model.Experimenter.
            Arguments:
            _cb -- The asynchronous callback object.
            user -- the experimenter
            active -- if true, then only shares which can be used for login will be returned. All draft shares (see {@code createShare}) and closed shares (see {@code closeShare}) will be filtered.
            current -- The Current object for the invocation.
            """
            pass

        def getMemberSharesFor_async(self, _cb, user, active, current=None):
            """
            Gets all shares where given
            omero.model.Experimenter is a member.
            Arguments:
            _cb -- The asynchronous callback object.
            user -- the experimenter
            active -- if true, then only shares which can be used for login will be returned. All draft shares (see {@code createShare}) and closed shares (see {@code closeShare}) will be filtered.
            current -- The Current object for the invocation.
            """
            pass

        def getContents_async(self, _cb, shareId, current=None):
            """
            Looks up all omero.model.IObject items belong to the
            omero.model.Session share.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            current -- The Current object for the invocation.
            """
            pass

        def getContentSubList_async(self, _cb, shareId, start, finish, current=None):
            """
            Returns a range of items from the share.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            start -- 
            finish -- 
            current -- The Current object for the invocation.
            """
            pass

        def getContentSize_async(self, _cb, shareId, current=None):
            """
            Returns the number of items in the share.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            current -- The Current object for the invocation.
            """
            pass

        def getContentMap_async(self, _cb, shareId, current=None):
            """
            Returns the contents of the share keyed by type.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            current -- The Current object for the invocation.
            """
            pass

        def createShare_async(self, _cb, description, expiration, items, exps, guests, enabled, current=None):
            """
            Creates omero.model.Session share with all related:
            omero.model.IObject itmes,
            omero.model.Experimenter members, and guests.
            Arguments:
            _cb -- The asynchronous callback object.
            description -- 
            expiration -- 
            items -- 
            exps -- 
            guests -- 
            enabled -- if true, then the share is immediately available for use. If false, then the share is in draft state. All methods on this interface will work for shares except {@code activate}. Similarly, the share password cannot be used by guests to login.
            current -- The Current object for the invocation.
            """
            pass

        def setDescription_async(self, _cb, shareId, description, current=None):
            pass

        def setExpiration_async(self, _cb, shareId, expiration, current=None):
            pass

        def setActive_async(self, _cb, shareId, active, current=None):
            pass

        def closeShare_async(self, _cb, shareId, current=None):
            """
            Closes omero.model.Session share. No further logins
            will be possible and all getters (e.g.
            {@code getMemberShares}, {@code getOwnShares}, ...) will
            filter these results if {@code onlyActive} is true.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            current -- The Current object for the invocation.
            """
            pass

        def addObjects_async(self, _cb, shareId, iobjects, current=None):
            """
            Adds new omero.model.IObject items to
            omero.model.Session share. Conceptually calls
            {@code addObjects} for every argument passed, but the
            graphs will be merged.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            iobjects -- 
            current -- The Current object for the invocation.
            """
            pass

        def addObject_async(self, _cb, shareId, iobject, current=None):
            """
            Adds new omero.model.IObject item to
            omero.model.Session share. The entire object graph
            with the exception of all Details will be loaded into the
            share. If you would like to load a single object, then pass
            an unloaded reference.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            iobject -- 
            current -- The Current object for the invocation.
            """
            pass

        def removeObjects_async(self, _cb, shareId, iobjects, current=None):
            """
            Remove existing items from the share.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            iobjects -- 
            current -- The Current object for the invocation.
            """
            pass

        def removeObject_async(self, _cb, shareId, iobject, current=None):
            """
            Removes existing omero.model.IObject object from the
            omero.model.Session share.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            iobject -- 
            current -- The Current object for the invocation.
            """
            pass

        def getCommentCount_async(self, _cb, shareIds, current=None):
            """
            Returns a map from share id to comment count.
            Arguments:
            _cb -- The asynchronous callback object.
            shareIds -- Not null.
            current -- The Current object for the invocation.
            Throws:
            ValidationException -- if a given share does not exist
            """
            pass

        def getComments_async(self, _cb, shareId, current=None):
            """
            Looks up all omero.model.Annotation comments which
            belong to the omero.model.Session share.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            current -- The Current object for the invocation.
            """
            pass

        def addComment_async(self, _cb, shareId, comment, current=None):
            """
            Creates omero.model.TextAnnotation comment for
            omero.model.Session share.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            comment -- 
            current -- The Current object for the invocation.
            """
            pass

        def addReply_async(self, _cb, shareId, comment, replyTo, current=None):
            """
            Creates omero.model.TextAnnotation comment which
            replies to an existing comment.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            comment -- 
            replyTo -- 
            current -- The Current object for the invocation.
            """
            pass

        def deleteComment_async(self, _cb, comment, current=None):
            """
            Deletes omero.model.Annotation comment from the
            database.
            Arguments:
            _cb -- The asynchronous callback object.
            comment -- 
            current -- The Current object for the invocation.
            """
            pass

        def getAllMembers_async(self, _cb, shareId, current=None):
            """
            Get all omero.model.Experimenter users who are a
            member of the share.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            current -- The Current object for the invocation.
            """
            pass

        def getAllGuests_async(self, _cb, shareId, current=None):
            """
            Get the email addresses for all share guests.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            current -- The Current object for the invocation.
            """
            pass

        def getAllUsers_async(self, _cb, shareId, current=None):
            """
            Get a single set containing the
            {@code omero.model.Experimenter.getOmeName} login names
            of the users as well email addresses for guests.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            current -- The Current object for the invocation.
            Throws:
            ValidationException -- if there is a conflict between email addresses and user names.
            """
            pass

        def addUsers_async(self, _cb, shareId, exps, current=None):
            """
            Adds omero.model.Experimenter experimenters to
            omero.model.Session share.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            exps -- 
            current -- The Current object for the invocation.
            """
            pass

        def addGuests_async(self, _cb, shareId, emailAddresses, current=None):
            """
            Adds guest email addresses to the share.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            emailAddresses -- 
            current -- The Current object for the invocation.
            """
            pass

        def removeUsers_async(self, _cb, shareId, exps, current=None):
            """
            Removes omero.model.Experimenter experimenters from
            omero.model.Session share.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            exps -- 
            current -- The Current object for the invocation.
            """
            pass

        def removeGuests_async(self, _cb, shareId, emailAddresses, current=None):
            """
            Removes guest email addresses from the share.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            emailAddresses -- 
            current -- The Current object for the invocation.
            """
            pass

        def addUser_async(self, _cb, shareId, exp, current=None):
            """
            Adds omero.model.Experimenter experimenter to
            omero.model.Session share.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            exp -- 
            current -- The Current object for the invocation.
            """
            pass

        def addGuest_async(self, _cb, shareId, emailAddress, current=None):
            """
            Adds guest email address to the share.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            emailAddress -- 
            current -- The Current object for the invocation.
            """
            pass

        def removeUser_async(self, _cb, shareId, exp, current=None):
            """
            Removes omero.model.Experimenter experimenter from
            omero.model.Session share.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            exp -- 
            current -- The Current object for the invocation.
            """
            pass

        def removeGuest_async(self, _cb, shareId, emailAddress, current=None):
            """
            Removes guest email address from share.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            emailAddress -- 
            current -- The Current object for the invocation.
            """
            pass

        def getActiveConnections_async(self, _cb, shareId, current=None):
            """
            Gets actual active connections to
            omero.model.Session share.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            current -- The Current object for the invocation.
            """
            pass

        def getPastConnections_async(self, _cb, shareId, current=None):
            """
            Gets previous connections to
            omero.model.Session share.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            current -- The Current object for the invocation.
            """
            pass

        def invalidateConnection_async(self, _cb, shareId, exp, current=None):
            """
            Makes the connection invalid for
            omero.model.Session share for specified user.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            exp -- 
            current -- The Current object for the invocation.
            """
            pass

        def getEvents_async(self, _cb, shareId, exp, _from, to, current=None):
            """
            Gets events for omero.model.Session share per
            omero.model.Experimenter experimenter for period of
            time.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            exp -- 
            _from -- 
            to -- 
            current -- The Current object for the invocation.
            """
            pass

        def notifyMembersOfShare_async(self, _cb, shareId, subject, message, html, current=None):
            """
            Notifies via email selected members of share.
            Arguments:
            _cb -- The asynchronous callback object.
            shareId -- 
            subject -- 
            message -- 
            html -- 
            current -- The Current object for the invocation.
            """
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.api._t_IShare)

        __repr__ = __str__

    _M_omero.api.ISharePrx = Ice.createTempClass()
    class ISharePrx(_M_omero.api.ServiceInterfacePrx):

        """
        Turns on the access control lists attached to the given
        share for the current session. Warning: this will slow down
        the execution of the current session for all database
        reads. Writing to the database will not be allowed. If
        share does not exist or is not accessible (non-members) or
        is disabled, then an omero.ValidationException is thrown.
        Arguments:
        shareId -- 
        _ctx -- The request context for the invocation.
        """
        def activate(self, shareId, _ctx=None):
            return _M_omero.api.IShare._op_activate.invoke(self, ((shareId, ), _ctx))

        """
        Turns on the access control lists attached to the given
        share for the current session. Warning: this will slow down
        the execution of the current session for all database
        reads. Writing to the database will not be allowed. If
        share does not exist or is not accessible (non-members) or
        is disabled, then an omero.ValidationException is thrown.
        Arguments:
        shareId -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_activate(self, shareId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_activate.begin(self, ((shareId, ), _response, _ex, _sent, _ctx))

        """
        Turns on the access control lists attached to the given
        share for the current session. Warning: this will slow down
        the execution of the current session for all database
        reads. Writing to the database will not be allowed. If
        share does not exist or is not accessible (non-members) or
        is disabled, then an omero.ValidationException is thrown.
        Arguments:
        shareId -- 
        """
        def end_activate(self, _r):
            return _M_omero.api.IShare._op_activate.end(self, _r)

        """
        Turns off the access control lists with the current share.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def deactivate(self, _ctx=None):
            return _M_omero.api.IShare._op_deactivate.invoke(self, ((), _ctx))

        """
        Turns off the access control lists with the current share.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_deactivate(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_deactivate.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Turns off the access control lists with the current share.
        Arguments:
        """
        def end_deactivate(self, _r):
            return _M_omero.api.IShare._op_deactivate.end(self, _r)

        """
        Gets a share as a omero.model.Session with all
        related: omero.model.Annotation comments,
        omero.model.Experimenter members, fully loaded.
        Unlike the other methods on this interface, if the
        sessionId is unknown, does not throw a
        omero.ValidationException.
        Arguments:
        shareId -- 
        _ctx -- The request context for the invocation.
        Returns: a omero.model.Session with id and omero.model.Details set or null. The owner in the Details object is the true owner, and the group in the Details has all member users linked. omero.model.Annotation instances of the share are linked to the omero.model.Session. Missing is a list of share guests.
        """
        def getShare(self, shareId, _ctx=None):
            return _M_omero.api.IShare._op_getShare.invoke(self, ((shareId, ), _ctx))

        """
        Gets a share as a omero.model.Session with all
        related: omero.model.Annotation comments,
        omero.model.Experimenter members, fully loaded.
        Unlike the other methods on this interface, if the
        sessionId is unknown, does not throw a
        omero.ValidationException.
        Arguments:
        shareId -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getShare(self, shareId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_getShare.begin(self, ((shareId, ), _response, _ex, _sent, _ctx))

        """
        Gets a share as a omero.model.Session with all
        related: omero.model.Annotation comments,
        omero.model.Experimenter members, fully loaded.
        Unlike the other methods on this interface, if the
        sessionId is unknown, does not throw a
        omero.ValidationException.
        Arguments:
        shareId -- 
        Returns: a omero.model.Session with id and omero.model.Details set or null. The owner in the Details object is the true owner, and the group in the Details has all member users linked. omero.model.Annotation instances of the share are linked to the omero.model.Session. Missing is a list of share guests.
        """
        def end_getShare(self, _r):
            return _M_omero.api.IShare._op_getShare.end(self, _r)

        """
        Returns a map from share id to the count of total members
        (including the owner). This is represented by
        omero.model.ShareMember links.
        Arguments:
        shareIds -- Not null.
        _ctx -- The request context for the invocation.
        Returns: Map with all ids present.
        Throws:
        ValidationException -- if a given share does not exist
        """
        def getMemberCount(self, shareIds, _ctx=None):
            return _M_omero.api.IShare._op_getMemberCount.invoke(self, ((shareIds, ), _ctx))

        """
        Returns a map from share id to the count of total members
        (including the owner). This is represented by
        omero.model.ShareMember links.
        Arguments:
        shareIds -- Not null.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getMemberCount(self, shareIds, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_getMemberCount.begin(self, ((shareIds, ), _response, _ex, _sent, _ctx))

        """
        Returns a map from share id to the count of total members
        (including the owner). This is represented by
        omero.model.ShareMember links.
        Arguments:
        shareIds -- Not null.
        Returns: Map with all ids present.
        Throws:
        ValidationException -- if a given share does not exist
        """
        def end_getMemberCount(self, _r):
            return _M_omero.api.IShare._op_getMemberCount.end(self, _r)

        """
        Gets all owned shares for the current
        omero.model.Experimenter.
        Arguments:
        active -- if true, then only shares which can be used for login will be returned. All draft shares (see {@code createShare}) and closed shares (see {@code closeShare}) will be filtered.
        _ctx -- The request context for the invocation.
        Returns: set of shares. Never null. May be empty.
        """
        def getOwnShares(self, active, _ctx=None):
            return _M_omero.api.IShare._op_getOwnShares.invoke(self, ((active, ), _ctx))

        """
        Gets all owned shares for the current
        omero.model.Experimenter.
        Arguments:
        active -- if true, then only shares which can be used for login will be returned. All draft shares (see {@code createShare}) and closed shares (see {@code closeShare}) will be filtered.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getOwnShares(self, active, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_getOwnShares.begin(self, ((active, ), _response, _ex, _sent, _ctx))

        """
        Gets all owned shares for the current
        omero.model.Experimenter.
        Arguments:
        active -- if true, then only shares which can be used for login will be returned. All draft shares (see {@code createShare}) and closed shares (see {@code closeShare}) will be filtered.
        Returns: set of shares. Never null. May be empty.
        """
        def end_getOwnShares(self, _r):
            return _M_omero.api.IShare._op_getOwnShares.end(self, _r)

        """
        Gets all shares where current
        omero.model.Experimenter is a member.
        Arguments:
        active -- if true, then only shares which can be used for login will be returned. All draft shares (see {@code createShare}) and closed shares (see {@code closeShare}) will be filtered.
        _ctx -- The request context for the invocation.
        Returns: set of shares. Never null. May be empty.
        """
        def getMemberShares(self, active, _ctx=None):
            return _M_omero.api.IShare._op_getMemberShares.invoke(self, ((active, ), _ctx))

        """
        Gets all shares where current
        omero.model.Experimenter is a member.
        Arguments:
        active -- if true, then only shares which can be used for login will be returned. All draft shares (see {@code createShare}) and closed shares (see {@code closeShare}) will be filtered.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getMemberShares(self, active, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_getMemberShares.begin(self, ((active, ), _response, _ex, _sent, _ctx))

        """
        Gets all shares where current
        omero.model.Experimenter is a member.
        Arguments:
        active -- if true, then only shares which can be used for login will be returned. All draft shares (see {@code createShare}) and closed shares (see {@code closeShare}) will be filtered.
        Returns: set of shares. Never null. May be empty.
        """
        def end_getMemberShares(self, _r):
            return _M_omero.api.IShare._op_getMemberShares.end(self, _r)

        """
        Gets all shares owned by the given
        omero.model.Experimenter.
        Arguments:
        user -- the experimenter
        active -- if true, then only shares which can be used for login will be returned. All draft shares (see {@code createShare}) and closed shares (see {@code closeShare}) will be filtered.
        _ctx -- The request context for the invocation.
        Returns: set of shares. Never null. May be empty.
        """
        def getSharesOwnedBy(self, user, active, _ctx=None):
            return _M_omero.api.IShare._op_getSharesOwnedBy.invoke(self, ((user, active), _ctx))

        """
        Gets all shares owned by the given
        omero.model.Experimenter.
        Arguments:
        user -- the experimenter
        active -- if true, then only shares which can be used for login will be returned. All draft shares (see {@code createShare}) and closed shares (see {@code closeShare}) will be filtered.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getSharesOwnedBy(self, user, active, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_getSharesOwnedBy.begin(self, ((user, active), _response, _ex, _sent, _ctx))

        """
        Gets all shares owned by the given
        omero.model.Experimenter.
        Arguments:
        user -- the experimenter
        active -- if true, then only shares which can be used for login will be returned. All draft shares (see {@code createShare}) and closed shares (see {@code closeShare}) will be filtered.
        Returns: set of shares. Never null. May be empty.
        """
        def end_getSharesOwnedBy(self, _r):
            return _M_omero.api.IShare._op_getSharesOwnedBy.end(self, _r)

        """
        Gets all shares where given
        omero.model.Experimenter is a member.
        Arguments:
        user -- the experimenter
        active -- if true, then only shares which can be used for login will be returned. All draft shares (see {@code createShare}) and closed shares (see {@code closeShare}) will be filtered.
        _ctx -- The request context for the invocation.
        Returns: set of shares. Never null. May be empty.
        """
        def getMemberSharesFor(self, user, active, _ctx=None):
            return _M_omero.api.IShare._op_getMemberSharesFor.invoke(self, ((user, active), _ctx))

        """
        Gets all shares where given
        omero.model.Experimenter is a member.
        Arguments:
        user -- the experimenter
        active -- if true, then only shares which can be used for login will be returned. All draft shares (see {@code createShare}) and closed shares (see {@code closeShare}) will be filtered.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getMemberSharesFor(self, user, active, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_getMemberSharesFor.begin(self, ((user, active), _response, _ex, _sent, _ctx))

        """
        Gets all shares where given
        omero.model.Experimenter is a member.
        Arguments:
        user -- the experimenter
        active -- if true, then only shares which can be used for login will be returned. All draft shares (see {@code createShare}) and closed shares (see {@code closeShare}) will be filtered.
        Returns: set of shares. Never null. May be empty.
        """
        def end_getMemberSharesFor(self, _r):
            return _M_omero.api.IShare._op_getMemberSharesFor.end(self, _r)

        """
        Looks up all omero.model.IObject items belong to the
        omero.model.Session share.
        Arguments:
        shareId -- 
        _ctx -- The request context for the invocation.
        Returns: list of objects. Not null. Probably not empty.
        """
        def getContents(self, shareId, _ctx=None):
            return _M_omero.api.IShare._op_getContents.invoke(self, ((shareId, ), _ctx))

        """
        Looks up all omero.model.IObject items belong to the
        omero.model.Session share.
        Arguments:
        shareId -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getContents(self, shareId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_getContents.begin(self, ((shareId, ), _response, _ex, _sent, _ctx))

        """
        Looks up all omero.model.IObject items belong to the
        omero.model.Session share.
        Arguments:
        shareId -- 
        Returns: list of objects. Not null. Probably not empty.
        """
        def end_getContents(self, _r):
            return _M_omero.api.IShare._op_getContents.end(self, _r)

        """
        Returns a range of items from the share.
        Arguments:
        shareId -- 
        start -- 
        finish -- 
        _ctx -- The request context for the invocation.
        """
        def getContentSubList(self, shareId, start, finish, _ctx=None):
            return _M_omero.api.IShare._op_getContentSubList.invoke(self, ((shareId, start, finish), _ctx))

        """
        Returns a range of items from the share.
        Arguments:
        shareId -- 
        start -- 
        finish -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getContentSubList(self, shareId, start, finish, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_getContentSubList.begin(self, ((shareId, start, finish), _response, _ex, _sent, _ctx))

        """
        Returns a range of items from the share.
        Arguments:
        shareId -- 
        start -- 
        finish -- 
        """
        def end_getContentSubList(self, _r):
            return _M_omero.api.IShare._op_getContentSubList.end(self, _r)

        """
        Returns the number of items in the share.
        Arguments:
        shareId -- 
        _ctx -- The request context for the invocation.
        """
        def getContentSize(self, shareId, _ctx=None):
            return _M_omero.api.IShare._op_getContentSize.invoke(self, ((shareId, ), _ctx))

        """
        Returns the number of items in the share.
        Arguments:
        shareId -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getContentSize(self, shareId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_getContentSize.begin(self, ((shareId, ), _response, _ex, _sent, _ctx))

        """
        Returns the number of items in the share.
        Arguments:
        shareId -- 
        """
        def end_getContentSize(self, _r):
            return _M_omero.api.IShare._op_getContentSize.end(self, _r)

        """
        Returns the contents of the share keyed by type.
        Arguments:
        shareId -- 
        _ctx -- The request context for the invocation.
        """
        def getContentMap(self, shareId, _ctx=None):
            return _M_omero.api.IShare._op_getContentMap.invoke(self, ((shareId, ), _ctx))

        """
        Returns the contents of the share keyed by type.
        Arguments:
        shareId -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getContentMap(self, shareId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_getContentMap.begin(self, ((shareId, ), _response, _ex, _sent, _ctx))

        """
        Returns the contents of the share keyed by type.
        Arguments:
        shareId -- 
        """
        def end_getContentMap(self, _r):
            return _M_omero.api.IShare._op_getContentMap.end(self, _r)

        """
        Creates omero.model.Session share with all related:
        omero.model.IObject itmes,
        omero.model.Experimenter members, and guests.
        Arguments:
        description -- 
        expiration -- 
        items -- 
        exps -- 
        guests -- 
        enabled -- if true, then the share is immediately available for use. If false, then the share is in draft state. All methods on this interface will work for shares except {@code activate}. Similarly, the share password cannot be used by guests to login.
        _ctx -- The request context for the invocation.
        """
        def createShare(self, description, expiration, items, exps, guests, enabled, _ctx=None):
            return _M_omero.api.IShare._op_createShare.invoke(self, ((description, expiration, items, exps, guests, enabled), _ctx))

        """
        Creates omero.model.Session share with all related:
        omero.model.IObject itmes,
        omero.model.Experimenter members, and guests.
        Arguments:
        description -- 
        expiration -- 
        items -- 
        exps -- 
        guests -- 
        enabled -- if true, then the share is immediately available for use. If false, then the share is in draft state. All methods on this interface will work for shares except {@code activate}. Similarly, the share password cannot be used by guests to login.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_createShare(self, description, expiration, items, exps, guests, enabled, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_createShare.begin(self, ((description, expiration, items, exps, guests, enabled), _response, _ex, _sent, _ctx))

        """
        Creates omero.model.Session share with all related:
        omero.model.IObject itmes,
        omero.model.Experimenter members, and guests.
        Arguments:
        description -- 
        expiration -- 
        items -- 
        exps -- 
        guests -- 
        enabled -- if true, then the share is immediately available for use. If false, then the share is in draft state. All methods on this interface will work for shares except {@code activate}. Similarly, the share password cannot be used by guests to login.
        """
        def end_createShare(self, _r):
            return _M_omero.api.IShare._op_createShare.end(self, _r)

        def setDescription(self, shareId, description, _ctx=None):
            return _M_omero.api.IShare._op_setDescription.invoke(self, ((shareId, description), _ctx))

        def begin_setDescription(self, shareId, description, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_setDescription.begin(self, ((shareId, description), _response, _ex, _sent, _ctx))

        def end_setDescription(self, _r):
            return _M_omero.api.IShare._op_setDescription.end(self, _r)

        def setExpiration(self, shareId, expiration, _ctx=None):
            return _M_omero.api.IShare._op_setExpiration.invoke(self, ((shareId, expiration), _ctx))

        def begin_setExpiration(self, shareId, expiration, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_setExpiration.begin(self, ((shareId, expiration), _response, _ex, _sent, _ctx))

        def end_setExpiration(self, _r):
            return _M_omero.api.IShare._op_setExpiration.end(self, _r)

        def setActive(self, shareId, active, _ctx=None):
            return _M_omero.api.IShare._op_setActive.invoke(self, ((shareId, active), _ctx))

        def begin_setActive(self, shareId, active, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_setActive.begin(self, ((shareId, active), _response, _ex, _sent, _ctx))

        def end_setActive(self, _r):
            return _M_omero.api.IShare._op_setActive.end(self, _r)

        """
        Closes omero.model.Session share. No further logins
        will be possible and all getters (e.g.
        {@code getMemberShares}, {@code getOwnShares}, ...) will
        filter these results if {@code onlyActive} is true.
        Arguments:
        shareId -- 
        _ctx -- The request context for the invocation.
        """
        def closeShare(self, shareId, _ctx=None):
            return _M_omero.api.IShare._op_closeShare.invoke(self, ((shareId, ), _ctx))

        """
        Closes omero.model.Session share. No further logins
        will be possible and all getters (e.g.
        {@code getMemberShares}, {@code getOwnShares}, ...) will
        filter these results if {@code onlyActive} is true.
        Arguments:
        shareId -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_closeShare(self, shareId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_closeShare.begin(self, ((shareId, ), _response, _ex, _sent, _ctx))

        """
        Closes omero.model.Session share. No further logins
        will be possible and all getters (e.g.
        {@code getMemberShares}, {@code getOwnShares}, ...) will
        filter these results if {@code onlyActive} is true.
        Arguments:
        shareId -- 
        """
        def end_closeShare(self, _r):
            return _M_omero.api.IShare._op_closeShare.end(self, _r)

        """
        Adds new omero.model.IObject items to
        omero.model.Session share. Conceptually calls
        {@code addObjects} for every argument passed, but the
        graphs will be merged.
        Arguments:
        shareId -- 
        iobjects -- 
        _ctx -- The request context for the invocation.
        """
        def addObjects(self, shareId, iobjects, _ctx=None):
            return _M_omero.api.IShare._op_addObjects.invoke(self, ((shareId, iobjects), _ctx))

        """
        Adds new omero.model.IObject items to
        omero.model.Session share. Conceptually calls
        {@code addObjects} for every argument passed, but the
        graphs will be merged.
        Arguments:
        shareId -- 
        iobjects -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_addObjects(self, shareId, iobjects, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_addObjects.begin(self, ((shareId, iobjects), _response, _ex, _sent, _ctx))

        """
        Adds new omero.model.IObject items to
        omero.model.Session share. Conceptually calls
        {@code addObjects} for every argument passed, but the
        graphs will be merged.
        Arguments:
        shareId -- 
        iobjects -- 
        """
        def end_addObjects(self, _r):
            return _M_omero.api.IShare._op_addObjects.end(self, _r)

        """
        Adds new omero.model.IObject item to
        omero.model.Session share. The entire object graph
        with the exception of all Details will be loaded into the
        share. If you would like to load a single object, then pass
        an unloaded reference.
        Arguments:
        shareId -- 
        iobject -- 
        _ctx -- The request context for the invocation.
        """
        def addObject(self, shareId, iobject, _ctx=None):
            return _M_omero.api.IShare._op_addObject.invoke(self, ((shareId, iobject), _ctx))

        """
        Adds new omero.model.IObject item to
        omero.model.Session share. The entire object graph
        with the exception of all Details will be loaded into the
        share. If you would like to load a single object, then pass
        an unloaded reference.
        Arguments:
        shareId -- 
        iobject -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_addObject(self, shareId, iobject, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_addObject.begin(self, ((shareId, iobject), _response, _ex, _sent, _ctx))

        """
        Adds new omero.model.IObject item to
        omero.model.Session share. The entire object graph
        with the exception of all Details will be loaded into the
        share. If you would like to load a single object, then pass
        an unloaded reference.
        Arguments:
        shareId -- 
        iobject -- 
        """
        def end_addObject(self, _r):
            return _M_omero.api.IShare._op_addObject.end(self, _r)

        """
        Remove existing items from the share.
        Arguments:
        shareId -- 
        iobjects -- 
        _ctx -- The request context for the invocation.
        """
        def removeObjects(self, shareId, iobjects, _ctx=None):
            return _M_omero.api.IShare._op_removeObjects.invoke(self, ((shareId, iobjects), _ctx))

        """
        Remove existing items from the share.
        Arguments:
        shareId -- 
        iobjects -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_removeObjects(self, shareId, iobjects, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_removeObjects.begin(self, ((shareId, iobjects), _response, _ex, _sent, _ctx))

        """
        Remove existing items from the share.
        Arguments:
        shareId -- 
        iobjects -- 
        """
        def end_removeObjects(self, _r):
            return _M_omero.api.IShare._op_removeObjects.end(self, _r)

        """
        Removes existing omero.model.IObject object from the
        omero.model.Session share.
        Arguments:
        shareId -- 
        iobject -- 
        _ctx -- The request context for the invocation.
        """
        def removeObject(self, shareId, iobject, _ctx=None):
            return _M_omero.api.IShare._op_removeObject.invoke(self, ((shareId, iobject), _ctx))

        """
        Removes existing omero.model.IObject object from the
        omero.model.Session share.
        Arguments:
        shareId -- 
        iobject -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_removeObject(self, shareId, iobject, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_removeObject.begin(self, ((shareId, iobject), _response, _ex, _sent, _ctx))

        """
        Removes existing omero.model.IObject object from the
        omero.model.Session share.
        Arguments:
        shareId -- 
        iobject -- 
        """
        def end_removeObject(self, _r):
            return _M_omero.api.IShare._op_removeObject.end(self, _r)

        """
        Returns a map from share id to comment count.
        Arguments:
        shareIds -- Not null.
        _ctx -- The request context for the invocation.
        Returns: Map with all ids present and 0 if no count exists.
        Throws:
        ValidationException -- if a given share does not exist
        """
        def getCommentCount(self, shareIds, _ctx=None):
            return _M_omero.api.IShare._op_getCommentCount.invoke(self, ((shareIds, ), _ctx))

        """
        Returns a map from share id to comment count.
        Arguments:
        shareIds -- Not null.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getCommentCount(self, shareIds, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_getCommentCount.begin(self, ((shareIds, ), _response, _ex, _sent, _ctx))

        """
        Returns a map from share id to comment count.
        Arguments:
        shareIds -- Not null.
        Returns: Map with all ids present and 0 if no count exists.
        Throws:
        ValidationException -- if a given share does not exist
        """
        def end_getCommentCount(self, _r):
            return _M_omero.api.IShare._op_getCommentCount.end(self, _r)

        """
        Looks up all omero.model.Annotation comments which
        belong to the omero.model.Session share.
        Arguments:
        shareId -- 
        _ctx -- The request context for the invocation.
        Returns: list of Annotation
        """
        def getComments(self, shareId, _ctx=None):
            return _M_omero.api.IShare._op_getComments.invoke(self, ((shareId, ), _ctx))

        """
        Looks up all omero.model.Annotation comments which
        belong to the omero.model.Session share.
        Arguments:
        shareId -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getComments(self, shareId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_getComments.begin(self, ((shareId, ), _response, _ex, _sent, _ctx))

        """
        Looks up all omero.model.Annotation comments which
        belong to the omero.model.Session share.
        Arguments:
        shareId -- 
        Returns: list of Annotation
        """
        def end_getComments(self, _r):
            return _M_omero.api.IShare._op_getComments.end(self, _r)

        """
        Creates omero.model.TextAnnotation comment for
        omero.model.Session share.
        Arguments:
        shareId -- 
        comment -- 
        _ctx -- The request context for the invocation.
        """
        def addComment(self, shareId, comment, _ctx=None):
            return _M_omero.api.IShare._op_addComment.invoke(self, ((shareId, comment), _ctx))

        """
        Creates omero.model.TextAnnotation comment for
        omero.model.Session share.
        Arguments:
        shareId -- 
        comment -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_addComment(self, shareId, comment, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_addComment.begin(self, ((shareId, comment), _response, _ex, _sent, _ctx))

        """
        Creates omero.model.TextAnnotation comment for
        omero.model.Session share.
        Arguments:
        shareId -- 
        comment -- 
        """
        def end_addComment(self, _r):
            return _M_omero.api.IShare._op_addComment.end(self, _r)

        """
        Creates omero.model.TextAnnotation comment which
        replies to an existing comment.
        Arguments:
        shareId -- 
        comment -- 
        replyTo -- 
        _ctx -- The request context for the invocation.
        Returns: the new omero.model.TextAnnotation
        """
        def addReply(self, shareId, comment, replyTo, _ctx=None):
            return _M_omero.api.IShare._op_addReply.invoke(self, ((shareId, comment, replyTo), _ctx))

        """
        Creates omero.model.TextAnnotation comment which
        replies to an existing comment.
        Arguments:
        shareId -- 
        comment -- 
        replyTo -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_addReply(self, shareId, comment, replyTo, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_addReply.begin(self, ((shareId, comment, replyTo), _response, _ex, _sent, _ctx))

        """
        Creates omero.model.TextAnnotation comment which
        replies to an existing comment.
        Arguments:
        shareId -- 
        comment -- 
        replyTo -- 
        Returns: the new omero.model.TextAnnotation
        """
        def end_addReply(self, _r):
            return _M_omero.api.IShare._op_addReply.end(self, _r)

        """
        Deletes omero.model.Annotation comment from the
        database.
        Arguments:
        comment -- 
        _ctx -- The request context for the invocation.
        """
        def deleteComment(self, comment, _ctx=None):
            return _M_omero.api.IShare._op_deleteComment.invoke(self, ((comment, ), _ctx))

        """
        Deletes omero.model.Annotation comment from the
        database.
        Arguments:
        comment -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_deleteComment(self, comment, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_deleteComment.begin(self, ((comment, ), _response, _ex, _sent, _ctx))

        """
        Deletes omero.model.Annotation comment from the
        database.
        Arguments:
        comment -- 
        """
        def end_deleteComment(self, _r):
            return _M_omero.api.IShare._op_deleteComment.end(self, _r)

        """
        Get all omero.model.Experimenter users who are a
        member of the share.
        Arguments:
        shareId -- 
        _ctx -- The request context for the invocation.
        """
        def getAllMembers(self, shareId, _ctx=None):
            return _M_omero.api.IShare._op_getAllMembers.invoke(self, ((shareId, ), _ctx))

        """
        Get all omero.model.Experimenter users who are a
        member of the share.
        Arguments:
        shareId -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getAllMembers(self, shareId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_getAllMembers.begin(self, ((shareId, ), _response, _ex, _sent, _ctx))

        """
        Get all omero.model.Experimenter users who are a
        member of the share.
        Arguments:
        shareId -- 
        """
        def end_getAllMembers(self, _r):
            return _M_omero.api.IShare._op_getAllMembers.end(self, _r)

        """
        Get the email addresses for all share guests.
        Arguments:
        shareId -- 
        _ctx -- The request context for the invocation.
        """
        def getAllGuests(self, shareId, _ctx=None):
            return _M_omero.api.IShare._op_getAllGuests.invoke(self, ((shareId, ), _ctx))

        """
        Get the email addresses for all share guests.
        Arguments:
        shareId -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getAllGuests(self, shareId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_getAllGuests.begin(self, ((shareId, ), _response, _ex, _sent, _ctx))

        """
        Get the email addresses for all share guests.
        Arguments:
        shareId -- 
        """
        def end_getAllGuests(self, _r):
            return _M_omero.api.IShare._op_getAllGuests.end(self, _r)

        """
        Get a single set containing the
        {@code omero.model.Experimenter.getOmeName} login names
        of the users as well email addresses for guests.
        Arguments:
        shareId -- 
        _ctx -- The request context for the invocation.
        Returns: a java.util.Set containing the login of all users
        Throws:
        ValidationException -- if there is a conflict between email addresses and user names.
        """
        def getAllUsers(self, shareId, _ctx=None):
            return _M_omero.api.IShare._op_getAllUsers.invoke(self, ((shareId, ), _ctx))

        """
        Get a single set containing the
        {@code omero.model.Experimenter.getOmeName} login names
        of the users as well email addresses for guests.
        Arguments:
        shareId -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getAllUsers(self, shareId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_getAllUsers.begin(self, ((shareId, ), _response, _ex, _sent, _ctx))

        """
        Get a single set containing the
        {@code omero.model.Experimenter.getOmeName} login names
        of the users as well email addresses for guests.
        Arguments:
        shareId -- 
        Returns: a java.util.Set containing the login of all users
        Throws:
        ValidationException -- if there is a conflict between email addresses and user names.
        """
        def end_getAllUsers(self, _r):
            return _M_omero.api.IShare._op_getAllUsers.end(self, _r)

        """
        Adds omero.model.Experimenter experimenters to
        omero.model.Session share.
        Arguments:
        shareId -- 
        exps -- 
        _ctx -- The request context for the invocation.
        """
        def addUsers(self, shareId, exps, _ctx=None):
            return _M_omero.api.IShare._op_addUsers.invoke(self, ((shareId, exps), _ctx))

        """
        Adds omero.model.Experimenter experimenters to
        omero.model.Session share.
        Arguments:
        shareId -- 
        exps -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_addUsers(self, shareId, exps, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_addUsers.begin(self, ((shareId, exps), _response, _ex, _sent, _ctx))

        """
        Adds omero.model.Experimenter experimenters to
        omero.model.Session share.
        Arguments:
        shareId -- 
        exps -- 
        """
        def end_addUsers(self, _r):
            return _M_omero.api.IShare._op_addUsers.end(self, _r)

        """
        Adds guest email addresses to the share.
        Arguments:
        shareId -- 
        emailAddresses -- 
        _ctx -- The request context for the invocation.
        """
        def addGuests(self, shareId, emailAddresses, _ctx=None):
            return _M_omero.api.IShare._op_addGuests.invoke(self, ((shareId, emailAddresses), _ctx))

        """
        Adds guest email addresses to the share.
        Arguments:
        shareId -- 
        emailAddresses -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_addGuests(self, shareId, emailAddresses, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_addGuests.begin(self, ((shareId, emailAddresses), _response, _ex, _sent, _ctx))

        """
        Adds guest email addresses to the share.
        Arguments:
        shareId -- 
        emailAddresses -- 
        """
        def end_addGuests(self, _r):
            return _M_omero.api.IShare._op_addGuests.end(self, _r)

        """
        Removes omero.model.Experimenter experimenters from
        omero.model.Session share.
        Arguments:
        shareId -- 
        exps -- 
        _ctx -- The request context for the invocation.
        """
        def removeUsers(self, shareId, exps, _ctx=None):
            return _M_omero.api.IShare._op_removeUsers.invoke(self, ((shareId, exps), _ctx))

        """
        Removes omero.model.Experimenter experimenters from
        omero.model.Session share.
        Arguments:
        shareId -- 
        exps -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_removeUsers(self, shareId, exps, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_removeUsers.begin(self, ((shareId, exps), _response, _ex, _sent, _ctx))

        """
        Removes omero.model.Experimenter experimenters from
        omero.model.Session share.
        Arguments:
        shareId -- 
        exps -- 
        """
        def end_removeUsers(self, _r):
            return _M_omero.api.IShare._op_removeUsers.end(self, _r)

        """
        Removes guest email addresses from the share.
        Arguments:
        shareId -- 
        emailAddresses -- 
        _ctx -- The request context for the invocation.
        """
        def removeGuests(self, shareId, emailAddresses, _ctx=None):
            return _M_omero.api.IShare._op_removeGuests.invoke(self, ((shareId, emailAddresses), _ctx))

        """
        Removes guest email addresses from the share.
        Arguments:
        shareId -- 
        emailAddresses -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_removeGuests(self, shareId, emailAddresses, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_removeGuests.begin(self, ((shareId, emailAddresses), _response, _ex, _sent, _ctx))

        """
        Removes guest email addresses from the share.
        Arguments:
        shareId -- 
        emailAddresses -- 
        """
        def end_removeGuests(self, _r):
            return _M_omero.api.IShare._op_removeGuests.end(self, _r)

        """
        Adds omero.model.Experimenter experimenter to
        omero.model.Session share.
        Arguments:
        shareId -- 
        exp -- 
        _ctx -- The request context for the invocation.
        """
        def addUser(self, shareId, exp, _ctx=None):
            return _M_omero.api.IShare._op_addUser.invoke(self, ((shareId, exp), _ctx))

        """
        Adds omero.model.Experimenter experimenter to
        omero.model.Session share.
        Arguments:
        shareId -- 
        exp -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_addUser(self, shareId, exp, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_addUser.begin(self, ((shareId, exp), _response, _ex, _sent, _ctx))

        """
        Adds omero.model.Experimenter experimenter to
        omero.model.Session share.
        Arguments:
        shareId -- 
        exp -- 
        """
        def end_addUser(self, _r):
            return _M_omero.api.IShare._op_addUser.end(self, _r)

        """
        Adds guest email address to the share.
        Arguments:
        shareId -- 
        emailAddress -- 
        _ctx -- The request context for the invocation.
        """
        def addGuest(self, shareId, emailAddress, _ctx=None):
            return _M_omero.api.IShare._op_addGuest.invoke(self, ((shareId, emailAddress), _ctx))

        """
        Adds guest email address to the share.
        Arguments:
        shareId -- 
        emailAddress -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_addGuest(self, shareId, emailAddress, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_addGuest.begin(self, ((shareId, emailAddress), _response, _ex, _sent, _ctx))

        """
        Adds guest email address to the share.
        Arguments:
        shareId -- 
        emailAddress -- 
        """
        def end_addGuest(self, _r):
            return _M_omero.api.IShare._op_addGuest.end(self, _r)

        """
        Removes omero.model.Experimenter experimenter from
        omero.model.Session share.
        Arguments:
        shareId -- 
        exp -- 
        _ctx -- The request context for the invocation.
        """
        def removeUser(self, shareId, exp, _ctx=None):
            return _M_omero.api.IShare._op_removeUser.invoke(self, ((shareId, exp), _ctx))

        """
        Removes omero.model.Experimenter experimenter from
        omero.model.Session share.
        Arguments:
        shareId -- 
        exp -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_removeUser(self, shareId, exp, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_removeUser.begin(self, ((shareId, exp), _response, _ex, _sent, _ctx))

        """
        Removes omero.model.Experimenter experimenter from
        omero.model.Session share.
        Arguments:
        shareId -- 
        exp -- 
        """
        def end_removeUser(self, _r):
            return _M_omero.api.IShare._op_removeUser.end(self, _r)

        """
        Removes guest email address from share.
        Arguments:
        shareId -- 
        emailAddress -- 
        _ctx -- The request context for the invocation.
        """
        def removeGuest(self, shareId, emailAddress, _ctx=None):
            return _M_omero.api.IShare._op_removeGuest.invoke(self, ((shareId, emailAddress), _ctx))

        """
        Removes guest email address from share.
        Arguments:
        shareId -- 
        emailAddress -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_removeGuest(self, shareId, emailAddress, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_removeGuest.begin(self, ((shareId, emailAddress), _response, _ex, _sent, _ctx))

        """
        Removes guest email address from share.
        Arguments:
        shareId -- 
        emailAddress -- 
        """
        def end_removeGuest(self, _r):
            return _M_omero.api.IShare._op_removeGuest.end(self, _r)

        """
        Gets actual active connections to
        omero.model.Session share.
        Arguments:
        shareId -- 
        _ctx -- The request context for the invocation.
        Returns: map of experimenter and IP address
        """
        def getActiveConnections(self, shareId, _ctx=None):
            return _M_omero.api.IShare._op_getActiveConnections.invoke(self, ((shareId, ), _ctx))

        """
        Gets actual active connections to
        omero.model.Session share.
        Arguments:
        shareId -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getActiveConnections(self, shareId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_getActiveConnections.begin(self, ((shareId, ), _response, _ex, _sent, _ctx))

        """
        Gets actual active connections to
        omero.model.Session share.
        Arguments:
        shareId -- 
        Returns: map of experimenter and IP address
        """
        def end_getActiveConnections(self, _r):
            return _M_omero.api.IShare._op_getActiveConnections.end(self, _r)

        """
        Gets previous connections to
        omero.model.Session share.
        Arguments:
        shareId -- 
        _ctx -- The request context for the invocation.
        Returns: map of experimenter and IP address
        """
        def getPastConnections(self, shareId, _ctx=None):
            return _M_omero.api.IShare._op_getPastConnections.invoke(self, ((shareId, ), _ctx))

        """
        Gets previous connections to
        omero.model.Session share.
        Arguments:
        shareId -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getPastConnections(self, shareId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_getPastConnections.begin(self, ((shareId, ), _response, _ex, _sent, _ctx))

        """
        Gets previous connections to
        omero.model.Session share.
        Arguments:
        shareId -- 
        Returns: map of experimenter and IP address
        """
        def end_getPastConnections(self, _r):
            return _M_omero.api.IShare._op_getPastConnections.end(self, _r)

        """
        Makes the connection invalid for
        omero.model.Session share for specified user.
        Arguments:
        shareId -- 
        exp -- 
        _ctx -- The request context for the invocation.
        """
        def invalidateConnection(self, shareId, exp, _ctx=None):
            return _M_omero.api.IShare._op_invalidateConnection.invoke(self, ((shareId, exp), _ctx))

        """
        Makes the connection invalid for
        omero.model.Session share for specified user.
        Arguments:
        shareId -- 
        exp -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_invalidateConnection(self, shareId, exp, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_invalidateConnection.begin(self, ((shareId, exp), _response, _ex, _sent, _ctx))

        """
        Makes the connection invalid for
        omero.model.Session share for specified user.
        Arguments:
        shareId -- 
        exp -- 
        """
        def end_invalidateConnection(self, _r):
            return _M_omero.api.IShare._op_invalidateConnection.end(self, _r)

        """
        Gets events for omero.model.Session share per
        omero.model.Experimenter experimenter for period of
        time.
        Arguments:
        shareId -- 
        exp -- 
        _from -- 
        to -- 
        _ctx -- The request context for the invocation.
        Returns: List of events
        """
        def getEvents(self, shareId, exp, _from, to, _ctx=None):
            return _M_omero.api.IShare._op_getEvents.invoke(self, ((shareId, exp, _from, to), _ctx))

        """
        Gets events for omero.model.Session share per
        omero.model.Experimenter experimenter for period of
        time.
        Arguments:
        shareId -- 
        exp -- 
        _from -- 
        to -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getEvents(self, shareId, exp, _from, to, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_getEvents.begin(self, ((shareId, exp, _from, to), _response, _ex, _sent, _ctx))

        """
        Gets events for omero.model.Session share per
        omero.model.Experimenter experimenter for period of
        time.
        Arguments:
        shareId -- 
        exp -- 
        _from -- 
        to -- 
        Returns: List of events
        """
        def end_getEvents(self, _r):
            return _M_omero.api.IShare._op_getEvents.end(self, _r)

        """
        Notifies via email selected members of share.
        Arguments:
        shareId -- 
        subject -- 
        message -- 
        html -- 
        _ctx -- The request context for the invocation.
        """
        def notifyMembersOfShare(self, shareId, subject, message, html, _ctx=None):
            return _M_omero.api.IShare._op_notifyMembersOfShare.invoke(self, ((shareId, subject, message, html), _ctx))

        """
        Notifies via email selected members of share.
        Arguments:
        shareId -- 
        subject -- 
        message -- 
        html -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_notifyMembersOfShare(self, shareId, subject, message, html, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IShare._op_notifyMembersOfShare.begin(self, ((shareId, subject, message, html), _response, _ex, _sent, _ctx))

        """
        Notifies via email selected members of share.
        Arguments:
        shareId -- 
        subject -- 
        message -- 
        html -- 
        """
        def end_notifyMembersOfShare(self, _r):
            return _M_omero.api.IShare._op_notifyMembersOfShare.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.api.ISharePrx.ice_checkedCast(proxy, '::omero::api::IShare', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.api.ISharePrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::api::IShare'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.api._t_ISharePrx = IcePy.defineProxy('::omero::api::IShare', ISharePrx)

    _M_omero.api._t_IShare = IcePy.defineClass('::omero::api::IShare', IShare, -1, (), True, False, None, (_M_omero.api._t_ServiceInterface,), ())
    IShare._ice_type = _M_omero.api._t_IShare

    IShare._op_activate = IcePy.Operation('activate', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0),), (), None, (_M_omero._t_ServerError,))
    IShare._op_deactivate = IcePy.Operation('deactivate', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), None, (_M_omero._t_ServerError,))
    IShare._op_getShare = IcePy.Operation('getShare', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0),), (), ((), _M_omero.model._t_Share, False, 0), (_M_omero._t_ServerError,))
    IShare._op_getMemberCount = IcePy.Operation('getMemberCount', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_omero.sys._t_LongList, False, 0),), (), ((), _M_omero.sys._t_CountMap, False, 0), (_M_omero._t_ServerError,))
    IShare._op_getOwnShares = IcePy.Operation('getOwnShares', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_bool, False, 0),), (), ((), _M_omero.api._t_SessionList, False, 0), (_M_omero._t_ServerError,))
    IShare._op_getMemberShares = IcePy.Operation('getMemberShares', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_bool, False, 0),), (), ((), _M_omero.api._t_SessionList, False, 0), (_M_omero._t_ServerError,))
    IShare._op_getSharesOwnedBy = IcePy.Operation('getSharesOwnedBy', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_omero.model._t_Experimenter, False, 0), ((), IcePy._t_bool, False, 0)), (), ((), _M_omero.api._t_SessionList, False, 0), (_M_omero._t_ServerError,))
    IShare._op_getMemberSharesFor = IcePy.Operation('getMemberSharesFor', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_omero.model._t_Experimenter, False, 0), ((), IcePy._t_bool, False, 0)), (), ((), _M_omero.api._t_SessionList, False, 0), (_M_omero._t_ServerError,))
    IShare._op_getContents = IcePy.Operation('getContents', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0),), (), ((), _M_omero.api._t_IObjectList, False, 0), (_M_omero._t_ServerError,))
    IShare._op_getContentSubList = IcePy.Operation('getContentSubList', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0)), (), ((), _M_omero.api._t_IObjectList, False, 0), (_M_omero._t_ServerError,))
    IShare._op_getContentSize = IcePy.Operation('getContentSize', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0),), (), ((), IcePy._t_int, False, 0), (_M_omero._t_ServerError,))
    IShare._op_getContentMap = IcePy.Operation('getContentMap', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0),), (), ((), _M_omero.api._t_IdListMap, False, 0), (_M_omero._t_ServerError,))
    IShare._op_createShare = IcePy.Operation('createShare', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_string, False, 0), ((), _M_omero._t_RTime, False, 0), ((), _M_omero.api._t_IObjectList, False, 0), ((), _M_omero.api._t_ExperimenterList, False, 0), ((), _M_omero.api._t_StringSet, False, 0), ((), IcePy._t_bool, False, 0)), (), ((), IcePy._t_long, False, 0), (_M_omero._t_ServerError,))
    IShare._op_setDescription = IcePy.Operation('setDescription', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0), ((), IcePy._t_string, False, 0)), (), None, (_M_omero._t_ServerError,))
    IShare._op_setExpiration = IcePy.Operation('setExpiration', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0), ((), _M_omero._t_RTime, False, 0)), (), None, (_M_omero._t_ServerError,))
    IShare._op_setActive = IcePy.Operation('setActive', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0), ((), IcePy._t_bool, False, 0)), (), None, (_M_omero._t_ServerError,))
    IShare._op_closeShare = IcePy.Operation('closeShare', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0),), (), None, (_M_omero._t_ServerError,))
    IShare._op_addObjects = IcePy.Operation('addObjects', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0), ((), _M_omero.api._t_IObjectList, False, 0)), (), None, (_M_omero._t_ServerError,))
    IShare._op_addObject = IcePy.Operation('addObject', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0), ((), _M_omero.model._t_IObject, False, 0)), (), None, (_M_omero._t_ServerError,))
    IShare._op_removeObjects = IcePy.Operation('removeObjects', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0), ((), _M_omero.api._t_IObjectList, False, 0)), (), None, (_M_omero._t_ServerError,))
    IShare._op_removeObject = IcePy.Operation('removeObject', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0), ((), _M_omero.model._t_IObject, False, 0)), (), None, (_M_omero._t_ServerError,))
    IShare._op_getCommentCount = IcePy.Operation('getCommentCount', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_omero.sys._t_LongList, False, 0),), (), ((), _M_omero.sys._t_CountMap, False, 0), (_M_omero._t_ServerError,))
    IShare._op_getComments = IcePy.Operation('getComments', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0),), (), ((), _M_omero.api._t_AnnotationList, False, 0), (_M_omero._t_ServerError,))
    IShare._op_addComment = IcePy.Operation('addComment', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0), ((), IcePy._t_string, False, 0)), (), ((), _M_omero.model._t_TextAnnotation, False, 0), (_M_omero._t_ServerError,))
    IShare._op_addReply = IcePy.Operation('addReply', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0), ((), IcePy._t_string, False, 0), ((), _M_omero.model._t_TextAnnotation, False, 0)), (), ((), _M_omero.model._t_TextAnnotation, False, 0), (_M_omero._t_ServerError,))
    IShare._op_deleteComment = IcePy.Operation('deleteComment', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), None, (_M_omero._t_ServerError,))
    IShare._op_getAllMembers = IcePy.Operation('getAllMembers', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0),), (), ((), _M_omero.api._t_ExperimenterList, False, 0), (_M_omero._t_ServerError,))
    IShare._op_getAllGuests = IcePy.Operation('getAllGuests', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0),), (), ((), _M_omero.api._t_StringSet, False, 0), (_M_omero._t_ServerError,))
    IShare._op_getAllUsers = IcePy.Operation('getAllUsers', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0),), (), ((), _M_omero.api._t_StringSet, False, 0), (_M_omero._t_ValidationException, _M_omero._t_ServerError))
    IShare._op_addUsers = IcePy.Operation('addUsers', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0), ((), _M_omero.api._t_ExperimenterList, False, 0)), (), None, (_M_omero._t_ServerError,))
    IShare._op_addGuests = IcePy.Operation('addGuests', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0), ((), _M_omero.api._t_StringSet, False, 0)), (), None, (_M_omero._t_ServerError,))
    IShare._op_removeUsers = IcePy.Operation('removeUsers', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0), ((), _M_omero.api._t_ExperimenterList, False, 0)), (), None, (_M_omero._t_ServerError,))
    IShare._op_removeGuests = IcePy.Operation('removeGuests', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0), ((), _M_omero.api._t_StringSet, False, 0)), (), None, (_M_omero._t_ServerError,))
    IShare._op_addUser = IcePy.Operation('addUser', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0), ((), _M_omero.model._t_Experimenter, False, 0)), (), None, (_M_omero._t_ServerError,))
    IShare._op_addGuest = IcePy.Operation('addGuest', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0), ((), IcePy._t_string, False, 0)), (), None, (_M_omero._t_ServerError,))
    IShare._op_removeUser = IcePy.Operation('removeUser', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0), ((), _M_omero.model._t_Experimenter, False, 0)), (), None, (_M_omero._t_ServerError,))
    IShare._op_removeGuest = IcePy.Operation('removeGuest', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0), ((), IcePy._t_string, False, 0)), (), None, (_M_omero._t_ServerError,))
    IShare._op_getActiveConnections = IcePy.Operation('getActiveConnections', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0),), (), ((), _M_omero.api._t_UserMap, False, 0), (_M_omero._t_ServerError,))
    IShare._op_getPastConnections = IcePy.Operation('getPastConnections', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0),), (), ((), _M_omero.api._t_UserMap, False, 0), (_M_omero._t_ServerError,))
    IShare._op_invalidateConnection = IcePy.Operation('invalidateConnection', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0), ((), _M_omero.model._t_Experimenter, False, 0)), (), None, (_M_omero._t_ServerError,))
    IShare._op_getEvents = IcePy.Operation('getEvents', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0), ((), _M_omero.model._t_Experimenter, False, 0), ((), _M_omero._t_RTime, False, 0), ((), _M_omero._t_RTime, False, 0)), (), ((), _M_omero.api._t_IObjectList, False, 0), (_M_omero._t_ServerError,))
    IShare._op_notifyMembersOfShare = IcePy.Operation('notifyMembersOfShare', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0), ((), IcePy._t_string, False, 0), ((), IcePy._t_string, False, 0), ((), IcePy._t_bool, False, 0)), (), None, (_M_omero._t_ServerError,))

    _M_omero.api.IShare = IShare
    del IShare

    _M_omero.api.ISharePrx = ISharePrx
    del ISharePrx

# End of module omero.api

__name__ = 'omero'

# End of module omero



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2007-2013 Glencoe Software, Inc. All rights reserved.
# Use is subject to license terms supplied in LICENSE.txt
"""
prefs plugin

Plugin read by omero.cli.Cli during initialization. The method(s)
defined here will be added to the Cli class for later use.

The pref plugin makes use of prefs.class from the common component.
"""
from __future__ import division

from builtins import zip
from builtins import str
from past.utils import old_div
import sys
import traceback
import os

from omero.cli import CLI
from omero.cli import BaseControl
from omero.cli import ExistingFile
from omero.cli import NonZeroReturnCode
from omero.config import ConfigXml
from omero.util import edit_path, get_omero_userdir
from omero.util.decorators import wraps
from omero.util.upgrade_check import UpgradeCheck
import portalocker
from omero_ext.argparse import SUPPRESS
from omero_ext.path import path

import omero.java

HELP = """Commands for server configuration

A config.xml file will be modified under your etc/grid directory. If you do
not have one, "upgrade" will create a new 4.2 configuration file.

The configuration values are used by `omero admin {start,deploy}` to set
properties on launch. See etc/grid/(win)default.xml. The "Profile" block
contains a reference to "__ACTIVE__" which is the current value in config.xml

By default, OMERO.grid will use the file in etc/grid/config.xml. If you would
like to configure your system to use $HOME/omero/config.xml, you will need to
modify the application descriptor.

Environment variables:
    OMERO_CONFIG - Changes the active profile

"""


def getprefs(args, dir):
    """
    Kept around temporarily for upgrading users from pre-4.2 configurations.
    """
    if not isinstance(args, list):
        raise Exception("Not a list")
    cmd = ["prefs"] + list(args)
    return omero.java.run(cmd, chdir=dir)


def _make_open_and_close_config(func, allow_readonly):
    def open_and_close_config(*args, **kwargs):
        args = list(args)
        self = args[0]
        argp = args[1]
        config = None
        if len(args) == 2:
            config = self.open_config(argp)
            if not allow_readonly:
                self.die_on_ro(config)
            args.append(config)
        try:
            return func(*args, **kwargs)
        finally:
            if config:
                config.close()
    return open_and_close_config


def with_config(func):
    """
    opens a config and passes it as the second argument.
    """
    return wraps(func)(_make_open_and_close_config(func, True))


def with_rw_config(func):
    """
    opens a config and passes it as the second argument.
    Requires that the returned config be writeable
    """
    return wraps(func)(_make_open_and_close_config(func, False))


class WriteableConfigControl(BaseControl):
    """
    Base class for controls which need write access to the OMERO configuration
    using the @with_rw_config decorator

    Note BaseControl should be used for read-only access using @with_config
    """

    def die_on_ro(self, config):
        if not config.save_on_close:
            self.ctx.die(333, "Cannot modify %s" % config.filename)

    def run_upgrade_check(self, config, agent):
        properties = config.as_map()
        upgrade_url = properties.get("omero.upgrades.url", None)
        if upgrade_url:
            uc = UpgradeCheck(agent, url=upgrade_url)
        else:
            uc = UpgradeCheck(agent)
        uc.run()
        return uc


class PrefsControl(WriteableConfigControl):

    def _configure(self, parser):
        parser.add_argument(
            "--source", help="Configuration file to be used. Default:"
            " etc/grid/config.xml")

        sub = parser.sub()

        parser.add(
            sub, self.all,
            "List all profiles in the current config.xml file.")

        default = sub.add_parser(
            "def", help="List (or set) the current active profile.",
            description="List (or set) the current active profile.")
        default.set_defaults(func=self.default)
        default.add_argument(
            "NAME", nargs="?",
            help="Name of the profile which should be made the new active"
            " profile.")

        list = parser.add(
            sub, self.list,
            "List all key-value pairs from the current profile (deprecated)")
        list.set_defaults(func=self.list)

        get = parser.add(
            sub, self.get,
            "Get key-value pairs from the current profile. All by default")
        get.set_defaults(func=self.get)
        get.add_argument(
            "KEY", nargs="*", help="Names of keys in the current profile")

        for x in [get, list]:
            secrets = x.add_mutually_exclusive_group()
            secrets.add_argument(
                "--show-password", action="store_true",
                help="Show values of sensitive keys (passwords, "
                "tokens, etc.) in the current profile")
            secrets.add_argument(
                "--hide-password", action="store_false",
                dest="show_password", help=SUPPRESS)

        set = parser.add(
            sub, self.set,
            "Set key-value pair in the current profile. Omit the"
            " value to remove the key.")
        append = parser.add(
            sub, self.append, "Append value to a key in the current profile.")
        remove = parser.add(
            sub, self.remove,
            "Remove value from a key in the current profile.")

        for x in [set, append, remove]:
            x.add_argument(
                "KEY", help="Name of the key in the current profile")
            # report is intended for use with cfg mgmt tools
            x.add_argument("--report", action="store_true",
                           help="Report if changes are made")
        set.add_argument(
            "-f", "--file", type=ExistingFile('r'),
            help="Load value from file")
        set.add_argument(
            "VALUE", nargs="?",
            help="Value to be set. If it is missing, the key will be removed")
        append.add_argument(
            "--set", action="store_true",
            help="Append only if not already in the list")
        append.add_argument("VALUE", help="Value to be appended")
        remove.add_argument("VALUE", help="Value to be removed")

        drop = parser.add(
            sub, self.drop, "Remove the profile from the configuration file")
        drop.add_argument("NAME", help="Name of the profile to remove")

        parser.add(sub, self.keys, "List all keys for the current profile")

        load = parser.add(
            sub, self.load,
            "Read into current profile from a file or standard input")
        load.add_argument(
            "-q", action="store_true", help="No error on conflict")
        load.add_argument(
            "file", nargs="*", type=ExistingFile('r'), default="-",
            help="Files to read from. Default to standard input if not"
            " specified")

        parse = parser.add(
            sub, self.parse,
            "Parse the configuration properties from the etc/omero.properties"
            " file and Web properties for readability.")
        parse.add_argument(
            "-f", "--file", type=ExistingFile('r'),
            help="Alternative location for a Java properties file")
        parse_group = parse.add_mutually_exclusive_group()
        parse_group.add_argument(
            "--defaults", action="store_true",
            help="Show key/value configuration defaults")
        parse_group.add_argument(
            "--rst", action="store_true",
            help="Generate reStructuredText from omero.properties")
        parse_group.add_argument(
            "--keys", action="store_true",
            help="Print just the keys from omero.properties")
        parse_group.add_argument(
            "--headers", action="store_true",
            help="Print all headers from omero.properties")
        parse.add_argument(
            "--no-web", action="store_true",
            help="Do not parse Web properties")

        parser.add(sub, self.edit, "Present the properties for the current"
                   " profile in your editor. Saving them will update your"
                   " profile.")
        parser.add(sub, self.version, "Print the configuration version for"
                   " the current profile.")
        parser.add(sub, self.path, "Print the file that is used for "
                   " configuration")
        parser.add(sub, self.lock, "Acquire the config file lock and hold"
                   " it")
        parser.add(sub, self.upgrade, "Create a 4.2 config.xml file based on"
                   " your current Java Preferences")

    def open_config(self, args):
        if args.source:
            cfg_xml = path(args.source)
            if not cfg_xml.exists():
                self.ctx.die(124, "File not found: %s" % args.source)
        else:
            if 'OMERODIR' in os.environ:
                base_dir = path(os.environ.get('OMERODIR'))
            else:
                self.ctx.die(125, 'FATAL: OMERODIR env variable not set')
            grid_dir = base_dir / "etc" / "grid"
            if not grid_dir.exists():
                self.ctx.err("%s not found; creating %s" %
                             (grid_dir, grid_dir))
                os.makedirs(grid_dir)
            cfg_xml = grid_dir / "config.xml"
        try:
            return ConfigXml(str(cfg_xml))
        except portalocker.LockException:
            self.ctx.die(112, "Could not acquire lock on %s" % cfg_xml)
        except Exception as e:
            self.ctx.die(113, str(e))

    def assert_valid_property_name(self, key):
        from re import search

        if search(r'[^A-Za-z0-9._-]', key):
            self.ctx.die(506, 'Illegal property name: {0}'.format(key))

    @with_config
    def all(self, args, config):
        for k, v in config.properties(None, True):
            self.ctx.out(k)

    @with_config
    def default(self, args, config):
        if args.NAME is not None:
            self.die_on_ro(config)
        self.ctx.out(config.default(args.NAME))

    @with_config
    def drop(self, args, config):
        try:
            config.remove(args.NAME)
        except KeyError:
            self.ctx.err("Unknown configuration: %s" % args.NAME)

    @with_config
    def list(self, args, config):
        self.ctx.err('WARNING: "config list" is deprecated, '
                     'use "config get" instead')
        args.KEY = []
        self.get(args, config)

    @with_config
    def get(self, args, config):
        orig = sorted(list(config.keys()))
        keys = sorted(list(args.KEY))
        if not keys:
            keys = orig
            for k in config.IGNORE:
                k in keys and keys.remove(k)

        is_password = (lambda x: x.lower().endswith('pass') or
                       x.lower().endswith('password'))
        for k in keys:
            if k not in orig:
                continue
            if args.KEY and len(args.KEY) == 1:
                self.ctx.out(config[k])
            else:
                if is_password(k) and not getattr(
                        args, "show_password", False):
                    self.ctx.out("%s=%s" % (k, '*' * 8 if config[k] else ''))
                else:
                    self.ctx.out("%s=%s" % (k, config[k]))

    @with_rw_config
    def set(self, args, config):
        if "=" in args.KEY:
            k, v = args.KEY.split("=", 1)
            msg = """ "=" in key name. Did you mean "...set %s %s"?"""
            if args.VALUE is None:
                k, v = args.KEY.split("=", 1)
                self.ctx.err(msg % (k, v))
            elif args.KEY.endswith("="):
                self.ctx.err(msg % (k, args.VALUE))
        elif args.file:
            if args.file == "-":
                # Read from standard input
                import fileinput
                f = fileinput.input(args.file)
            else:
                f = args.file
            try:
                self.assert_valid_property_name(args.KEY)
                config[args.KEY] = (''.join(f)).rstrip()
            finally:
                f.close()
        elif args.VALUE is None:
            del config[args.KEY]
            if args.report:
                self.ctx.out('Changed: Removed %s' % args.KEY)
        else:
            self.assert_valid_property_name(args.KEY)
            config[args.KEY] = args.VALUE
            if args.report:
                self.ctx.out('Changed: Set %s:%s' % (args.KEY, args.VALUE))

    def get_list_value(self, args, config):
        import json
        try:
            list_value = json.loads(config[args.KEY])
        except ValueError:
            self.ctx.die(510, "No JSON object could be decoded")

        if not isinstance(list_value, list):
            self.ctx.die(511, "Property %s is not a list" % args.KEY)
        return list_value

    def get_omeroweb_default(self, key):
        try:
            from omeroweb import settings
            setting = settings.CUSTOM_SETTINGS_MAPPINGS.get(key)
            default = setting[2](setting[1]) if setting else []
        except Exception as e:
            self.ctx.dbg(traceback.format_exc())
            self.ctx.die(514,
                         "Cannot retrieve default value for property %s: %s" %
                         (key, e))
        if not isinstance(default, list):
            self.ctx.die(515, "Property %s is not a list" % key)
        return default

    @with_rw_config
    def append(self, args, config):
        import json
        if args.KEY in list(config.keys()):
            list_value = self.get_list_value(args, config)
        elif args.KEY.startswith('omero.web.'):
            list_value = self.get_omeroweb_default(args.KEY)
        else:
            list_value = []
        jv = json.loads(args.VALUE)
        if not args.set or jv not in list_value:
            list_value.append(json.loads(args.VALUE))
            config[args.KEY] = json.dumps(list_value)
            if args.report:
                self.ctx.out(
                    'Changed: Appended %s:%s' % (args.KEY, args.VALUE))

    @with_rw_config
    def remove(self, args, config):
        if args.KEY not in list(config.keys()):
            if args.KEY.startswith('omero.web.'):
                list_value = self.get_omeroweb_default(args.KEY)
            else:
                self.ctx.die(512, "Property %s is not defined" % (args.KEY))
        else:
            list_value = self.get_list_value(args, config)
        import json
        if json.loads(args.VALUE) not in list_value:
            self.ctx.die(513, "%s is not defined in %s"
                         % (args.VALUE, args.KEY))

        list_value.remove(json.loads(args.VALUE))
        config[args.KEY] = json.dumps(list_value)
        if args.report:
            self.ctx.out('Changed: Removed %s:%s' % (args.KEY, args.VALUE))

    @with_config
    def keys(self, args, config):
        for k in sorted(config.keys()):
            if k not in config.IGNORE:
                self.ctx.out(k)

    def parse(self, args):
        from omero.install.config_parser import PropertyParser
        pp = PropertyParser()
        if args.file:
            args.file.close()
            pp.parse_file(str(path(args.file.name).abspath()))
        else:
            pp.parse_lines(self.ctx.get_config_property_lines(self.dir))

        # Parse PSQL profile file
        for p in pp:
            if p.key == "omero.db.profile":
                psql_file = self.dir / "etc" / "profiles" / p.val
                pp.parse_file(str(psql_file.abspath()))
                break

        # Parse OMERO.web configuration properties
        if not args.no_web:
            pp.parse_module('omeroweb.settings')

        # Display options
        if args.headers:
            pp.print_headers()
        elif args.keys:
            pp.print_keys()
        elif args.rst:
            pp.print_rst()
        else:
            pp.print_defaults()

    @with_rw_config
    def load(self, args, config):
        keys = None
        if not args.q:
            keys = list(config.keys())

        # Handle all lines before updating config in case of error.
        new_config = dict(config)
        try:
            for f in args.file:
                if f == "-":
                    # Read from standard input
                    import fileinput
                    f = fileinput.input(f)

                try:
                    previous = None
                    for line in f:
                        if previous:
                            line = previous + line
                        previous = self.handle_line(line, new_config, keys)
                finally:
                    if f != "-":
                        f.close()
        except NonZeroReturnCode:
            raise
        except Exception as e:
            self.ctx.dbg(traceback.format_exc())
            self.ctx.die(968, "Cannot read %s: %s" % (args.file, e))
        for key, value in list(new_config.items()):
            config[key] = value

    @with_rw_config
    def edit(self, args, config, edit_path=edit_path):
        from omero.util.temp_files import create_path, remove_path
        start_text = "# Edit your preferences below. Comments are ignored\n"
        for k in sorted(config.keys()):
            start_text += ("%s=%s\n" % (k, config[k]))
        temp_file = create_path()
        try:
            edit_path(temp_file, start_text)
        except RuntimeError as re:
            self.ctx.dbg(traceback.format_exc())
            self.ctx.die(954, "%s: Failed to edit %s"
                         % (getattr(re, "pid", "Unknown"), temp_file))
        args.NAME = config.default()
        old_config = dict(config)
        self.drop(args, config)
        args.file = [open(str(temp_file), "r")]
        args.q = True
        try:
            self.load(args, config)
        except Exception as e:
            for key, value in list(old_config.items()):
                config[key] = value
            raise e
        finally:
            remove_path(temp_file)

    @with_config
    def version(self, args, config):
        self.ctx.out(config.version(config.default()))

    @with_config
    def path(self, args, config):
        self.ctx.out(config.filename)

    @with_rw_config
    def lock(self, args, config):
        self.ctx.input("Press enter to unlock")

    @with_rw_config
    def upgrade(self, args, config):
        self.ctx.out("Importing pre-4.2 preferences")
        txt = getprefs(["get"], str(old_div(self.ctx.dir, "lib")))

        # Handle all lines before updating config in case of error.
        new_config = dict(config)
        for line in txt.split("\n"):
            self.handle_line(line, new_config, None)
        for key, value in list(new_config.items()):
            config[key] = value

        # Upgrade procedure for 4.2
        MSG = """Manually modify them via "omero config old set ..." and \
re-run"""
        m = config.as_map()
        for x in ("keyStore", "keyStorePassword", "trustStore",
                  "trustStorePassword"):
            old = "omero.ldap." + x
            new = "omero.security." + x
            if old in m:
                config[new] = config[old]

        attributes, values = [], []
        if "omero.ldap.attributes" in m:
            attributes = config["omero.ldap.attributes"]
            attributes = attributes.split(",")
        if "omero.ldap.values" in m:
            values = config["omero.ldap.values"]
            values = values.split(",")

        if len(attributes) != len(values):
            raise ValueError("%s != %s\nLDAP properties in pre-4.2"
                             " configuration are invalid.\n%s"
                             % (attributes, values, MSG))
        pairs = list(zip(attributes, values))
        if pairs:
            if len(pairs) == 1:
                user_filter = "(%s=%s)" % (tuple(pairs[0]))
            else:
                user_filter = "(&%s)" % ["(%s=%s)" % tuple(pair)
                                         for pair in pairs]
            config["omero.ldap.user_filter"] = user_filter

        if "omero.ldap.groups" in m:
            raise ValueError("Not currently handling omero.ldap.groups\n%s"
                             % MSG)

        config["omero.config.upgraded"] = "4.2.0"

    def handle_line(self, line, config, keys):
        line = line.strip()
        if not line or line.startswith("#"):
            return None
        if line.endswith("\\"):
            return line[:-1]

        parts = line.split("=", 1)
        if len(parts[0]) == 0:
            return
        if len(parts) == 1:
            parts.append("")

        _key = parts[0]
        _new = parts[1]
        if _key in list(config.keys()):
            _old = config[_key]
        else:
            self.assert_valid_property_name(_key)
            _old = None

        if keys and _key in keys and _new != _old:

            self.ctx.die(502, "Duplicate property: %s ('%s' => '%s')"
                         % (_key, _old, _new))
            keys.append(_key)

        config[_key] = _new

    def old(self, args):
        self.ctx.out(getprefs(args.target, str(old_div(self.ctx.dir, "lib"))))

try:
    register("config", PrefsControl, HELP)
except NameError:
    if __name__ == "__main__":
        cli = CLI()
        cli.register("config", PrefsControl, HELP)
        cli.invoke(sys.argv[1:])
